import { FC, ReactNode, useEffect, useImperativeHandle, useMemo, useRef, useState } from 'react'
import { InternalShoppingCartAction, ShoppingCartAction, ShoppingContext, useShoppingCart } from '../../context'
import { ColorMeta, ColorStore, ShoppingContextValue } from '../../context'

export type TriggerCheckboxOptions = { colorStore: ColorStore; changedCheckbox: ColorMeta }

export interface ShoppingCartPropsType {
  initialValues?: ColorStore
  onTriggerCheckbox?: (options: TriggerCheckboxOptions) => void
  children?: ReactNode
  ref?: ShoppingCartAction
}
// 购物车上下文组件
export const ShoppingCart: FC<ShoppingCartPropsType> = (props) => {
  const { children, onTriggerCheckbox, initialValues, ref: refProps } = props
  console.log('重新渲染 ShoppingCart context', refProps)

  const defaultShoppingCart = useShoppingCart()

  const ref = (refProps || defaultShoppingCart) as InternalShoppingCartAction

  const onTriggerCheckboxRef = useRef(onTriggerCheckbox)
  onTriggerCheckboxRef.current = onTriggerCheckbox

  const [colorStore, setColorStore] = useState<ColorStore>(() => initialValues || {})
  const [changedCheckbox, setChangedCheckbox] = useState<ColorMeta>({})
  // 当前高亮的客户
  const [currentCheckedPurchaserId, setCurrentCheckedPurchaserId] = useState<number>(-1)
  // 是否管理状态
  const [isManageStatus, setManageStatus] = useState(false)

  const ctx: ShoppingContextValue = useMemo(() => {
    console.log('useMemo ShoppingContextValue')
    return {
      isManageStatus,
      setManageStatus,
      currentCheckedPurchaserId, // 当前高亮的客户
      setCurrentCheckedPurchaserId, // 设置高亮当前客户
      colorStore,
      setChangedCheckbox(color) {
        setColorStore((prev) => ({ ...prev, [color.id as number]: color }))
        setChangedCheckbox(color)
      },
    }
  }, [colorStore, currentCheckedPurchaserId, isManageStatus])
  // 暴露给外界
  useImperativeHandle(
    ref.__INTERNAL__,
    () => ({
      getManageStatus() {
        return ctx.isManageStatus
      },
      setManageStatus: ctx.setManageStatus,
    }),
    [ctx.isManageStatus, ctx.setManageStatus],
  )

  // 这里要在 useEffect 也就是刷新 state 后再调用，否则如果在 onFieldsChangeRef 修改值会覆盖掉上次修改
  useEffect(() => {
    onTriggerCheckboxRef.current?.({
      colorStore,
      changedCheckbox,
    })
  }, [colorStore, changedCheckbox])

  return <ShoppingContext.Provider value={ctx}>{children}</ShoppingContext.Provider>
}
