import React, { useRef } from 'react'
import { useContext } from 'react'

/**
 * 456: {
 *  id: 456,
 *  checked: true
 * },
 * 457: {
 *  id: 457,
 *  checked: true
 * }
 */
export type ColorStore = Record<string, any>

export interface ColorMeta {
  id?: number
  checked?: boolean
}

export interface ShoppingContextValue {
  isManageStatus: boolean
  setManageStatus: (isManageStatus: boolean) => void
  currentCheckedPurchaserId: number
  setCurrentCheckedPurchaserId: (purchaserId: number) => void
  colorStore: ColorStore
  setChangedCheckbox: (color: ColorMeta) => void
}

export const ShoppingContext = React.createContext<ShoppingContextValue | null>(null)

export function useShoppingContext() {
  const ctx = useContext(ShoppingContext)
  if (!ctx) {
    throw new Error('没有获取到shopping context')
  }
  return ctx
}

export function useWatch(id: ColorMeta['id']) {
  const { colorStore } = useShoppingContext()
  console.log('colorStore==>', colorStore)
  return id ? colorStore[id] : colorStore
}

export interface ShoppingCartAction {
  setManageStatus: (manageStatus: boolean) => void
  getManageStatus: () => boolean
}

export interface InternalShoppingCartAction extends ShoppingCartAction {
  __INTERNAL__: React.MutableRefObject<ShoppingCartAction | null>
}

function throwError(): never {
  throw new Error('有没有用 ref 这个props？')
}

export function useShoppingCart(): ShoppingCartAction {
  const __INTERNAL__ = useRef<ShoppingCartAction | null>(null)
  return {
    __INTERNAL__,
    setManageStatus(manageStatus: boolean) {
      const action = __INTERNAL__.current
      if (!action) {
        throwError()
      }
      action.setManageStatus(manageStatus)
    },
    getManageStatus() {
      const action = __INTERNAL__.current
      if (!action) {
        throwError()
      }
      return action.getManageStatus()
    },
  } as InternalShoppingCartAction
}
