import Search from '@/components/search'
import { View } from '@tarojs/components'
import Taro, { useDidShow } from '@tarojs/taro'
import React, { FC, useCallback, useContext, useEffect, useMemo, useRef, useState } from 'react'
import styles from './index.module.scss'
import classnames from 'classnames'
import IconText from '@/components/iconText'
import InfiniteScroll from '@/components/infiniteScroll'
import ItemList from './components/shoppingCartItem/index'
import BottomBtns from '@/components/BottomBtns'
import { formatPriceDiv } from '@/common/format'
import BottomSettleBar from './components/bottomSettleBar'
import BottomEditBar from './components/bottomEditBar'
import { ShoppingCartListApi } from '@/api/index'
import { dataLoadingStatus } from '@/common/util'
import { ShoppingCart, TriggerCheckboxOptions } from './components/shoppingCart/index'
import {  useShoppingCart, useShoppingContext } from './context'

export const Shopping: FC = () => {

  const [checkboxData, setCheckboxData] = useState<Partial<TriggerCheckboxOptions>>({})
  
  const handleTriggerCheckbox = ({ colorStore, changedCheckbox }) => {
    console.log('handleTriggerCheckbox==>', colorStore, changedCheckbox)
    setCheckboxData({ colorStore, changedCheckbox })
  }
  return (
    <ShoppingCart onTriggerCheckbox={handleTriggerCheckbox}>
      <ShoppingCartContainer checkboxData={checkboxData} />
    </ShoppingCart>
  )
}

interface InternalContainer {
  checkboxData?: Partial<TriggerCheckboxOptions>
}

const ShoppingCartContainer:FC<InternalContainer> = (props) => {

  const { checkboxData } = props

  const { isManageStatus, setManageStatus } = useShoppingContext()

  //输入了搜索关键字
  const getSearchData = useCallback((e) => {
    // pageNum.current.page = 1
    // setOrderData(() => ({ list: [], total: 0 }))
    // setSearchField((val) => ({ ...val, name: e, size: 10 }))
  }, [])

  const shoppingCart = useShoppingCart()
  console.log('shoppingCart==>', shoppingCart)

  // 管理
  const onStartToManage = () => {
    setManageStatus(!isManageStatus)
    // shoppingCart.setManageStatus(!isManageStatus)
  }

  const [selectedAmount, setSelectedAmount] = useState(0)

  // 结算
  const handleSettleAccount = () => {}
  // 删除
  const handleDelete = () => {}

  const handleSelectAllCheckbox = (isSelectAll: boolean) => {
    console.log('handleSelectAllCheckbox', isSelectAll)
  }

  const [listHeight, setListHeight] = useState('auto')

  const { fetchData, state } = ShoppingCartListApi()

  const [shoppingCartData, setShoppingCartData] = useState<{ list: ShoppingCartData[]; total: number }>({ list: [], total: 0 })

  //数据加载状态
  const statusMore = useMemo(() => {
    return dataLoadingStatus({ list: shoppingCartData.list, total: shoppingCartData.total, status: state.loading })
  }, [shoppingCartData, state])

  useDidShow(() => {
    ;(async () => {
      await fetchData()
      let query = Taro.createSelectorQuery()
      console.log('query', query)
      query.select('#shoppingContainer').boundingClientRect()
      query.select('#topBar').boundingClientRect()
      query.select('#bottomBar').boundingClientRect()
      query.exec((res) => {
        console.log('res==>', res)
        const containerHeight = res[0].height
        const topBarHeight = res[1].height
        const bottomBarHeight = res[2].height
        const listHeight = containerHeight - topBarHeight - bottomBarHeight
        setListHeight(listHeight + 'px')
      })
    })()
  })

  useEffect(() => {
    setShoppingCartData({ list: state.data, total: state.data.length })
  }, [state])

  

  return (
    <View className={classnames('flex-col', styles.shopping)} id='shoppingContainer'>
      <View className={styles['shopping--topBar']} id='topBar'>
        <Search placeholder='请输入客户名称' showBtn={false} changeOnSearch={getSearchData} debounceTime={300}>
          <View className={styles.flexBox} onClick={onStartToManage}>
            {isManageStatus ? (
              <IconText iconName='icon-guanlidingdan' text='取消' customClass={styles['icon--manage--cancel']} />
            ) : (
              <IconText iconName='icon-guanlidingdan' text='管理' />
            )}
          </View>
        </Search>
      </View>
      <View className={classnames('flex-item', 'flex-col', styles['shopping--context'])}>
        <View className={classnames(styles.shopping__list__container, 'flex-item')} style={{ height: listHeight }}>
          <InfiniteScroll statusMore={statusMore}>
            {!!shoppingCartData?.list?.length &&
              shoppingCartData?.list?.map((item, index) => {
                return <ItemList itemData={item} key={index}></ItemList>
              })}
          </InfiniteScroll>
        </View>
        <View id='bottomBar'>
          {isManageStatus ? (
            <BottomEditBar onDelete={handleDelete} onSelectCheckbox={handleSelectAllCheckbox}></BottomEditBar>
          ) : (
            <BottomSettleBar onSettleAccount={handleSettleAccount} amount={selectedAmount}></BottomSettleBar>
          )}
        </View>
      </View>
    </View>
  )
}

export default Shopping
